@@ -1,14 +1,5 @@
-# Copyright REFITT Team 2019. All rights reserved.
-#
-# This program is free software: you can redistribute it and/or modify it under the
-# terms of the Apache License (v2.0) as published by the Apache Software Foundation.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT ANY
-# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-# PARTICULAR PURPOSE. See the Apache License for more details.
-#
-# You should have received a copy of the Apache License along with this program.
-# If not, see <https://www.apache.org/licenses/LICENSE-2.0>.
+# SPDX-FileCopyrightText: 2021 REFITT Team
+# SPDX-License-Identifier: Apache-2.0
 
 """Generate/update credentials for a user."""
 
@@ -32,6 +23,7 @@ from rich.syntax import Syntax
 # internal libs
 from ...database.model import User, Client, Session, NotFound, DEFAULT_EXPIRE_TIME, DEFAULT_CLIENT_LEVEL
 from ...core.exceptions import log_exception
+from ...web.token import Cipher
 
 # public interface
 __all__ = ['AuthApp', ]
@@ -40,7 +32,8 @@ __all__ = ['AuthApp', ]
 PROGRAM = 'refitt auth'
 PADDING = ' ' * len(PROGRAM)
 USAGE = f"""\
-usage: {PROGRAM} [-h] <user> <action> [<options>...]
+usage: {PROGRAM} <user> {{--gen-key [--level NUM] | --gen-secret | --gen-token [--expires SECONDS]}}
+       {PROGRAM} [-h] --gen-rootkey
 {__doc__}\
 """
 
@@ -55,11 +48,12 @@ actions:
     --gen-key                Generate all new credentials.
     --gen-secret             Generate new client secret.
     --gen-token              Generate new session token.
+    --gen-rootkey            Generate new rootkey.
     --revoke                 Revoke credentials.
     
 options:
-    --expires     SEC        Seconds until token expires (default: {DEFAULT_EXPIRE_TIME}).
-    --level       NUM        Apply specific authorization level (default: {DEFAULT_CLIENT_LEVEL}).
+-e, --expires     SEC        Seconds until token expires (default: {DEFAULT_EXPIRE_TIME}).
+-l, --level       NUM        Apply specific authorization level (default: {DEFAULT_CLIENT_LEVEL}).
 -h, --help                   Show this message and exit.\
 """
 
@@ -74,16 +68,18 @@ class AuthApp(Application):
     interface = Interface(PROGRAM, USAGE, HELP)
 
     user: Union[int, str] = None
-    interface.add_argument('user')
+    interface.add_argument('user', nargs='?', default=None)
 
     gen_key: bool = False
     gen_secret: bool = False
     gen_token: bool = False
     revoke_all: bool = False
+    gen_rootkey: bool = False
     action_interface = interface.add_mutually_exclusive_group()
     action_interface.add_argument('--gen-key', action='store_true')
     action_interface.add_argument('--gen-secret', action='store_true')
     action_interface.add_argument('--gen-token', action='store_true')
+    action_interface.add_argument('--gen-rootkey', action='store_true')
     action_interface.add_argument('--revoke', action='store_true', dest='revoke_all')
 
     level: Optional[int] = DEFAULT_CLIENT_LEVEL
@@ -109,16 +105,18 @@ class AuthApp(Application):
 
     def check_arguments(self) -> None:
         """Additional logical validation of arguments."""
-        actions = 'gen_key', 'gen_secret', 'gen_token', 'revoke_all'
-        actions_specified = map(functools.partial(getattr, self), actions)  # noqa: typing?
+        actions = 'gen_key', 'gen_secret', 'gen_token', 'gen_rootkey', 'revoke_all'
+        actions_specified = map(functools.partial(getattr, self), actions)
         if not any(actions_specified):
             raise ArgumentError(f'Must specify action')
+        if not self.gen_rootkey and not self.user:
+            raise ArgumentError('Must specify <user>')
 
     def format_output(self, data: Dict[str, str]):
         """Format and print credentials."""
         if self.format_json:
             if sys.stdout.isatty():
-                Console().print(Syntax(json.dumps(data, indent=4), 'json',
+                Console().print(Syntax(json.dumps(data, indent=4), 'json', theme='solarized-dark',
                                        word_wrap=True, background_color='default'))
             else:
                 print(json.dumps(data, indent=4), file=sys.stdout, flush=True)
@@ -134,6 +132,7 @@ class AuthApp(Application):
             'gen_key': self.update_key,
             'gen_secret': self.update_secret,
             'gen_token': self.update_token,
+            'gen_rootkey': self.create_rootkey,
             'revoke_all': self.revoke_credentials
         }
 
@@ -172,6 +171,11 @@ class AuthApp(Application):
         token = Session.new(self.user_id, expires=self.expires)
         return {'token': token.encrypt()}
 
+    @staticmethod
+    def create_rootkey() -> Dict[str, str]:
+        """Generate a new rootkey."""
+        return {'rootkey': Cipher.new_rootkey().value, }
+
     @property
     def user_id(self) -> int:
         """The user's ID (lookup based on `alias` if necessary)."""
