@@ -1,14 +1,5 @@
-# Copyright REFITT Team 2019. All rights reserved.
-#
-# This program is free software: you can redistribute it and/or modify it under the
-# terms of the Apache License (v2.0) as published by the Apache Software Foundation.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT ANY
-# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-# PARTICULAR PURPOSE. See the Apache License for more details.
-#
-# You should have received a copy of the Apache License along with this program.
-# If not, see <https://www.apache.org/licenses/LICENSE-2.0>.
+# SPDX-FileCopyrightText: 2021 REFITT Team
+# SPDX-License-Identifier: Apache-2.0
 
 """
 Send electronic mail.
@@ -35,6 +26,10 @@ from email.utils import formatdate
 # external libs
 from pandas import read_csv
 
+# public interface
+__all__ = ['UserAuth', 'Mail', 'MailServer', 'MailTemplate', 'TestMail', 'RecommendationMail',
+           'templates', 'TEMPLATES', ]
+
 
 # initialize module level logger
 log = logging.getLogger(__name__)
