@@ -3,6 +3,7 @@
 
 """Implements the common `Alert` interface for all broker clients."""
 
+
 # type annotations
 from __future__ import annotations
 from typing import List, Dict, Union, Any, Optional
@@ -14,7 +15,7 @@ from abc import ABC, abstractmethod
 
 # internal libs
 from ...database.model import ObjectType, Object, Source, ObservationType, Observation, Alert
-from ...database.core import Session
+from ...database.interface import Session
 
 # public interface
 __all__ = ['AlertInterface', ]
