@@ -1,7 +1,7 @@
 # SPDX-FileCopyrightText: 2021 REFITT Team
 # SPDX-License-Identifier: Apache-2.0
 
-"""Define AntaresAlert and AntaresClient"""
+"""Define AntaresAlert and AntaresClient."""
 
 
 # type annotations
