@@ -1,7 +1,7 @@
 # SPDX-FileCopyrightText: 2021 REFITT Team
 # SPDX-License-Identifier: Apache-2.0
 
-"""Core database ORM definitions."""
+"""Core database model definitions."""
 
 
 # type annotations
@@ -9,6 +9,7 @@ from __future__ import annotations
 from typing import List, Tuple, Dict, Any, Type, Optional, Callable, TypeVar, Union
 
 # standard libs
+import re
 import random
 import logging
 from base64 import encodebytes as base64_encode, decodebytes as base64_decode
@@ -18,29 +19,24 @@ from functools import lru_cache
 # external libs
 from names_generator.names import LEFT, RIGHT
 from sqlalchemy import Column, ForeignKey, Index, func, type_coerce, or_
-from sqlalchemy.exc import IntegrityError
+from sqlalchemy.ext.declarative import declared_attr, declarative_base
 from sqlalchemy.orm import relationship, aliased, Query
-from sqlalchemy.orm.exc import NoResultFound, MultipleResultsFound
+from sqlalchemy.exc import IntegrityError, NoResultFound, MultipleResultsFound
 from sqlalchemy.types import Integer, BigInteger, DateTime, Float, Text, String, JSON, Boolean, LargeBinary
+from sqlalchemy.schema import Sequence, CheckConstraint
 from sqlalchemy.dialects.postgresql import JSONB
 
 # internal libs
-from .core import schema, Session as _Session
+from .interface import schema, config, Session as _Session
 from ..web.token import Key, Secret, Token, JWT
 
-# NOTE: declarative base is imported from StreamKit's ORM.
-# This does not relate to the engine/session itself and we don't do anything with them.
-# It helps to have them share a base with our tables for initialization.
-from streamkit.database.core.orm import Table as Base
-from streamkit.database.core.orm import Level, Topic, Host, Message, Subscriber, Access
-
 # public interface
-__all__ = ['Base', 'DatabaseError', 'NotFound', 'NotDistinct', 'AlreadyExists', 'IntegrityError',
-           'Level', 'Topic', 'Host', 'Subscriber', 'Message', 'Access',
+__all__ = ['DatabaseError', 'NotFound', 'NotDistinct', 'AlreadyExists', 'IntegrityError',
+           'ModelInterface', 'Level', 'Topic', 'Host', 'Subscriber', 'Message', 'Access',
            'User', 'Facility', 'FacilityMap', 'ObjectType', 'Object', 'SourceType',
            'Source', 'ObservationType', 'Observation', 'Forecast', 'Alert', 'FileType', 'File',
            'RecommendationTag', 'RecommendationGroup', 'Recommendation', 'ModelType', 'Model',
-           'Client', 'Session', 'tables', 'indices', ]
+           'Client', 'Session', 'tables', 'indices', 'DEFAULT_EXPIRE_TIME', 'DEFAULT_CLIENT_LEVEL', ]
 
 
 # initialize module level logger
@@ -48,7 +44,7 @@ log = logging.getLogger(__name__)
 
 
 class DatabaseError(Exception):
-    """Generic error with respect to the database model."""
+    """Generic error with respect to the db model."""
 
 
 class NotFound(NoResultFound):
@@ -63,7 +59,6 @@ class AlreadyExists(DatabaseError):
     """Exception specific to a record with unique properties already existing."""
 
 
-# JSON value types and their coerced type before loading into database
 __NT = type(None)
 __VT = TypeVar('__VT', __NT, bool, int, float, str, Dict[str, Any], List[str])
 __RT = Union[__VT, datetime, bytes]
@@ -103,7 +98,6 @@ def __dump_bytes(value: __RT) -> __VT:
         return base64_encode(value).decode().strip().split('\n')
 
 
-# list of defined type coercion filters
 __LM = Callable[[__VT], __RT]
 __DM = Callable[[__RT], __VT]
 __loaders: List[__LM] = [__load_datetime, __load_bytes, ]
@@ -115,7 +109,7 @@ def __load_imp(value: __VT, filters: List[__LM]) -> __RT:
 
 
 def _load(value: __VT) -> __RT:
-    """Passively coerce value types of stored record assets to database compatible types."""
+    """Passively coerce value types of stored record assets to db compatible types."""
     return __load_imp(value, __loaders)
 
 
@@ -124,15 +118,25 @@ def __dump_imp(value: __RT, filters: List[__DM]) -> __VT:
 
 
 def _dump(value: __RT) -> __VT:
-    """Passively coerce database types to JSON encoded types."""
+    """Passively coerce db types to JSON encoded types."""
     return __dump_imp(value, __dumpers)
 
 
-class CoreMixin:
+class ModelBase:
     """Core mixin class for all models."""
 
+    @declared_attr
+    def __tablename__(cls) -> str:  # noqa: cls
+        """The table name should be the "snake_case" of the "ClassName"."""
+        return re.sub(r'(?<!^)(?=[A-Z])', '_', cls.__name__).lower()
+
+    @declared_attr
+    def __table_args__(cls) -> Dict[str, Any]:  # noqa: cls
+        """Common table attributes."""
+        return {'schema': schema, }
+
     columns: Dict[str, type] = {}
-    relationships: Dict[str, Type[Base]] = {}
+    relationships: Dict[str, Type[ModelInterface]] = {}
 
     def __repr__(self) -> str:
         """String representation of record."""
@@ -149,12 +153,17 @@ class CoreMixin:
         return {name: getattr(self, name) for name in self.columns}
 
     @classmethod
-    def from_dict(cls: Type[Base], data: Dict[str, Any]) -> Base:
+    def from_dict(cls: Type[ModelInterface], data: Dict[str, Any]) -> ModelInterface:
         """Build record from existing dictionary."""
         return cls(**data)
 
     @classmethod
-    def from_json(cls: Type[Base], data: Dict[str, __VT]) -> Base:
+    def new(cls: Type[Model], **fields) -> ModelInterface:
+        """Create new instance of the model with default fields."""
+        return cls(**fields)
+
+    @classmethod
+    def from_json(cls: Type[ModelInterface], data: Dict[str, __VT]) -> ModelInterface:
         """Build record from JSON data (already loaded as dictionary)."""
         return cls.from_dict({k: _load(v) for k, v in data.items()})
 
@@ -172,8 +181,8 @@ class CoreMixin:
         return data
 
     @classmethod
-    def from_id(cls: Type[Base], id: int, session: _Session = None) -> Base:
-        """Query for record using unique `id` if applicable."""
+    def from_id(cls: Type[ModelInterface], id: int, session: _Session = None) -> ModelInterface:
+        """Query using unique `id`."""
         try:
             if hasattr(cls, 'id'):
                 session = session or _Session()
@@ -184,13 +193,13 @@ class CoreMixin:
             raise cls.NotFound(f'No {cls.__tablename__} with id={id}') from error
 
     @classmethod
-    def add(cls: Type[Base], data: dict, session: _Session = None) -> Base:
+    def add(cls: Type[ModelInterface], data: dict, session: _Session = None) -> ModelInterface:
         """Add record from existing `data`, return constructed record."""
         record, = cls.add_all([data, ], session=session)
         return record
 
     @classmethod
-    def add_all(cls: Type[Base], data: List[dict], session: _Session = None) -> List[Base]:
+    def add_all(cls: Type[ModelInterface], data: List[dict], session: _Session = None) -> List[ModelInterface]:
         """Add list of new records to the database and return constructed records."""
         session = session or _Session()
         try:
@@ -205,76 +214,46 @@ class CoreMixin:
             raise
 
     @classmethod
-    def update(cls: Type[Base], id: int, session: Session = None, **data) -> Base:
+    def update(cls: Type[ModelInterface], id: int, **data) -> ModelInterface:
         """Update named attributes of specified record."""
-        session = session or _Session()
         try:
-            record = cls.from_id(id, session)
+            record = cls.from_id(id)
             for field, value in data.items():
                 if field in cls.columns:
                     setattr(record, field, value)
                 else:
                     record.data = {**record.data, field: value}
-            session.commit()
+            _Session.commit()
             log.info(f'Updated {cls.__tablename__} ({id})')
             return record
         except (IntegrityError, DatabaseError):
-            session.rollback()
+            _Session.rollback()
             raise
 
     @classmethod
-    def delete(cls: Type[Base], id: int, session: _Session = None) -> None:
+    def delete(cls: Type[ModelInterface], id: int) -> None:
         """Delete existing record with `id`."""
-        session = session or _Session()
-        record = cls.from_id(id, session)
-        session.delete(record)
-        session.commit()
+        record = cls.from_id(id)
+        _Session.delete(record)
+        _Session.commit()
         log.info(f'Deleted {cls.__tablename__} ({id})')
 
     @classmethod
-    def count(cls, session: _Session = None) -> int:
+    def count(cls) -> int:
         """Count of records in table."""
-        session = session or _Session()
-        return session.query(cls).count()
+        return cls.query().count()
 
     @classmethod
-    def query(cls: Type[Base]) -> Query:
+    def query(cls: Type[ModelInterface]) -> Query:
         return _Session.query(cls)
 
 
-# NOTE: patch existing table definitions.
-# This is a hack and will be redone at a later point (likely improvement to StreamKit).
-Level.from_json = lambda data: Level(**{k: _load(v) for k, v in data.items()})
-Topic.from_json = lambda data: Topic(**{k: _load(v) for k, v in data.items()})
-Host.from_json = lambda data: Host(**{k: _load(v) for k, v in data.items()})
-Message.from_json = lambda data: Message(**{k: _load(v) for k, v in data.items()})
-Subscriber.from_json = lambda data: Subscriber(**{k: _load(v) for k, v in data.items()})
-Access.from_json = lambda data: Access(**{k: _load(v) for k, v in data.items()})
-Level.to_tuple = Level.values
-Topic.to_tuple = Topic.values
-Host.to_tuple = Host.values
-Message.to_tuple = Message.values
-Subscriber.to_tuple = Subscriber.values
-Access.to_tuple = Access.values
-Level.columns = {'id': int, 'name': str}
-Level.relationships = {}
-Topic.columns = {'id': int, 'name': str}
-Topic.relationships = {}
-Host.columns = {'id': int, 'name': str}
-Host.relationships = {}
-Subscriber.columns = {'id': int, 'name': str}
-Subscriber.relationships = {}
-Access.columns = {'subscriber_id': int, 'topic_id': int, 'time': datetime}
-Access.relationships = {'subscriber': Subscriber, 'topic': Topic}
-Message.columns = {'id': int, 'time': datetime, 'topic_id': int, 'level_id': int, 'host_id': int, 'text': str}
-Message.relationships = {'topic': Topic, 'level': Level, 'host': Host}
+# declarative base inherits common interface
+ModelInterface = declarative_base(cls=ModelBase)
 
 
-class User(Base, CoreMixin):
-    """User profiles store characteristics about a person."""
-
-    __tablename__ = 'user'
-    __table_args__ = {'schema': schema}
+class User(ModelInterface):
+    """User profiles store attributes about a participating human observer."""
 
     id = Column('id', Integer(), primary_key=True, nullable=False)
     first_name = Column('first_name', Text(), nullable=False)
@@ -363,12 +342,9 @@ class User(Base, CoreMixin):
         log.info(f'Deleted user ({user_id})')
 
 
-class Facility(Base, CoreMixin):
+class Facility(ModelInterface):
     """Facility profiles store characteristics about a telescope and it's instruments."""
 
-    __tablename__ = 'facility'
-    __table_args__ = {'schema': schema}
-
     id = Column('id', Integer(), primary_key=True, nullable=False)
     name = Column('name', Text(), unique=True, nullable=False)
     latitude = Column('latitude', Float(), nullable=False)
@@ -441,12 +417,9 @@ class Facility(Base, CoreMixin):
         log.info(f'Deleted facility ({facility_id})')
 
 
-class FacilityMap(Base, CoreMixin):
+class FacilityMap(ModelInterface):
     """Mapping table between users and facilities."""
 
-    __tablename__ = 'facility_map'
-    __table_args__ = {'schema': schema}
-
     user_id = Column('user_id', Integer(), ForeignKey(User.id, ondelete='cascade'),
                      primary_key=True, nullable=False)
     facility_id = Column('facility_id', Integer(), ForeignKey(Facility.id, ondelete='cascade'),
@@ -458,7 +431,7 @@ class FacilityMap(Base, CoreMixin):
     }
 
     @classmethod
-    def from_id(cls: Type[Base], id: int, session: _Session = None) -> Base:
+    def from_id(cls, id: int, session: _Session = None) -> FacilityMap:
         raise NotImplementedError()
 
     @classmethod
@@ -474,17 +447,13 @@ class FacilityMap(Base, CoreMixin):
         raise NotImplementedError()
 
 
-# New credentials will be initialized with this level unless
-# otherwise specified
+# New credentials will be initialized with this level unless otherwise specified
 DEFAULT_CLIENT_LEVEL: int = 10
 
 
-class Client(Base, CoreMixin):
+class Client(ModelInterface):
     """Client stores user authorization and authentication."""
 
-    __tablename__ = 'client'
-    __table_args__ = {'schema': schema}
-
     id = Column('id', Integer(), primary_key=True, nullable=False)
     user_id = Column('user_id', Integer(), ForeignKey(User.id), unique=True, nullable=False)
     level = Column('level', Integer(), nullable=False)
@@ -529,16 +498,7 @@ class Client(Base, CoreMixin):
 
     @classmethod
     def new(cls, user_id: int, level: int = DEFAULT_CLIENT_LEVEL) -> Tuple[Key, Secret, Client]:
-        """
-        Create client credentials for `user` with `level`.
-
-        Args:
-            user_id (int or `User`):
-                An existing user.
-
-            level (int):
-                Authorization level (default: `DEFAULT_CLIENT_LEVEL`).
-        """
+        """Create client credentials for `user_id` with `level`."""
         session = _Session()
         user = User.from_id(user_id, session)
         key, secret = Key.generate(), Secret.generate()
@@ -572,17 +532,13 @@ class Client(Base, CoreMixin):
         return key, secret
 
 
-# New session tokens if not otherwise requested will have
-# the following lifetime (seconds)
-DEFAULT_EXPIRE_TIME: int = 900  # 15 minutes
+# New session tokens if not otherwise requested will have the following lifetime (seconds)
+DEFAULT_EXPIRE_TIME: int = 900  # i.e., 15 minutes
 
 
-class Session(Base, CoreMixin):
+class Session(ModelInterface):
     """Session stores hashed token with claim details."""
 
-    __tablename__ = 'session'
-    __table_args__ = {'schema': schema}
-
     id = Column('id', Integer(), primary_key=True, nullable=False)
     client_id = Column('client_id', Integer(), ForeignKey(Client.id, ondelete='cascade'), unique=True, nullable=False)
     expires = Column('expires', DateTime(timezone=True), nullable=True)  # NULL is no-expiration!
@@ -637,12 +593,9 @@ class Session(Base, CoreMixin):
         return jwt
 
 
-class ObjectType(Base, CoreMixin):
+class ObjectType(ModelInterface):
     """Object types (e.g., 'SNIa')."""
 
-    __tablename__ = 'object_type'
-    __table_args__ = {'schema': schema}
-
     id = Column('id', Integer(), primary_key=True, nullable=False)
     name = Column('name', Text(), unique=True, nullable=False)
     description = Column('description', Text(), nullable=False)
@@ -666,11 +619,8 @@ class ObjectType(Base, CoreMixin):
             raise ObjectType.NotFound(f'No object_type with name={name}') from error
 
 
-class Object(Base, CoreMixin):
-    """Object table."""
-
-    __tablename__ = 'object'
-    __table_args__ = {'schema': schema}
+class Object(ModelInterface):
+    """An astronomical object defines names, position, and other attributes."""
 
     id = Column('id', Integer(), primary_key=True, nullable=False)
     type_id = Column('type_id', Integer(), ForeignKey(ObjectType.id), nullable=False)
@@ -730,12 +680,9 @@ class Object(Base, CoreMixin):
             raise
 
 
-class ObservationType(Base, CoreMixin):
+class ObservationType(ModelInterface):
     """Observation types (e.g., 'g-ztf')."""
 
-    __tablename__ = 'observation_type'
-    __table_args__ = {'schema': schema}
-
     id = Column('id', Integer(), primary_key=True, nullable=False)
     name = Column('name', Text(), unique=True, nullable=False)
     units = Column('units', Text(), nullable=False)
@@ -761,12 +708,9 @@ class ObservationType(Base, CoreMixin):
             raise ObservationType.NotFound(f'No observation_type with name={name}') from error
 
 
-class SourceType(Base, CoreMixin):
+class SourceType(ModelInterface):
     """Source types (e.g., 'broker')."""
 
-    __tablename__ = 'source_type'
-    __table_args__ = {'schema': schema}
-
     id = Column('id', Integer(), primary_key=True, nullable=False)
     name = Column('name', Text(), unique=True, nullable=False)
     description = Column('description', Text(), nullable=False)
@@ -790,12 +734,9 @@ class SourceType(Base, CoreMixin):
             raise SourceType.NotFound(f'No source_type with name={name}') from error
 
 
-class Source(Base, CoreMixin):
+class Source(ModelInterface):
     """Source table."""
 
-    __tablename__ = 'source'
-    __table_args__ = {'schema': schema}
-
     id = Column('id', Integer(), primary_key=True, nullable=False)
     type_id = Column('type_id', Integer(), ForeignKey(SourceType.id), nullable=False)
     facility_id = Column('facility_id', Integer(), ForeignKey(Facility.id), nullable=True)
@@ -863,12 +804,9 @@ class Source(Base, CoreMixin):
                                'description': f'Observer (alias={user.alias}, facility={facility.name})'})
 
 
-class Observation(Base, CoreMixin):
+class Observation(ModelInterface):
     """Observation table."""
 
-    __tablename__ = 'observation'
-    __table_args__ = {'schema': schema}
-
     id = Column('id', Integer().with_variant(BigInteger(), 'postgresql'), primary_key=True, nullable=False)
     type_id = Column('type_id', Integer(), ForeignKey(ObservationType.id), nullable=False)
     object_id = Column('object_id', Integer(), ForeignKey(Object.id), nullable=False)
@@ -917,12 +855,9 @@ observation_time_index = Index('observation_time_index', Observation.time)
 observation_recorded_index = Index('observation_recorded_index', Observation.recorded)
 
 
-class Alert(Base, CoreMixin):
+class Alert(ModelInterface):
     """Alert table."""
 
-    __tablename__ = 'alert'
-    __table_args__ = {'schema': schema}
-
     id = Column('id', Integer().with_variant(BigInteger(), 'postgresql'), primary_key=True, nullable=False)
     observation_id = Column('observation_id', Integer().with_variant(BigInteger(), 'postgresql'),
                             ForeignKey(Observation.id), unique=True, nullable=False)
@@ -950,12 +885,9 @@ class Alert(Base, CoreMixin):
             raise Alert.NotFound(f'No alert with observation_id={observation_id}') from error
 
 
-class FileType(Base, CoreMixin):
+class FileType(ModelInterface):
     """File type table."""
 
-    __tablename__ = 'file_type'
-    __table_args__ = {'schema': schema}
-
     id = Column('id', Integer(), primary_key=True, nullable=False)
     name = Column('name', Text(), unique=True, nullable=False)
     description = Column('description', Text(), nullable=False)
@@ -984,12 +916,9 @@ class FileType(Base, CoreMixin):
         return [file_type.name for file_type in cls.query().all()]
 
 
-class File(Base, CoreMixin):
+class File(ModelInterface):
     """File table."""
 
-    __tablename__ = 'file'
-    __table_args__ = {'schema': schema}
-
     id = Column('id', Integer().with_variant(BigInteger(), 'postgresql'), primary_key=True, nullable=False)
     observation_id = Column('observation_id', Integer().with_variant(BigInteger(), 'postgresql'),
                             ForeignKey(Observation.id), unique=True, nullable=False)
@@ -1020,12 +949,9 @@ class File(Base, CoreMixin):
             raise File.NotFound(f'No file with observation_id={observation_id}') from error
 
 
-class Forecast(Base, CoreMixin):
+class Forecast(ModelInterface):
     """Forecast table."""
 
-    __tablename__ = 'forecast'
-    __table_args__ = {'schema': schema}
-
     id = Column('id', Integer().with_variant(BigInteger(), 'postgresql'), primary_key=True, nullable=False)
     observation_id = Column('observation_id', Integer().with_variant(BigInteger(), 'postgresql'),
                             ForeignKey(Observation.id), unique=True, nullable=False)
@@ -1053,12 +979,9 @@ class Forecast(Base, CoreMixin):
             raise Forecast.NotFound(f'No forecast with observation_id={observation_id}') from error
 
 
-class RecommendationGroup(Base, CoreMixin):
+class RecommendationGroup(ModelInterface):
     """Recommendation group table."""
 
-    __tablename__ = 'recommendation_group'
-    __table_args__ = {'schema': schema}
-
     id = Column('id', Integer(), primary_key=True, nullable=False)
     created = Column('created', DateTime(timezone=True), nullable=False, server_default=func.now())
 
@@ -1087,12 +1010,9 @@ class RecommendationGroup(Base, CoreMixin):
         return cls.query().order_by(cls.id.desc()).filter(cls.id <= cls.latest().id - offset).limit(limit).all()
 
 
-class RecommendationTag(Base, CoreMixin):
+class RecommendationTag(ModelInterface):
     """Recommendation tag table."""
 
-    __tablename__ = 'recommendation_tag'
-    __table_args__ = {'schema': schema}
-
     id = Column('id', Integer(), primary_key=True, nullable=False)
     object_id = Column('object_id', Integer(), ForeignKey(Object.id), unique=True, nullable=False)
     name = Column('name', Text(), unique=True, nullable=True)
@@ -1160,12 +1080,9 @@ class RecommendationTag(Base, CoreMixin):
         return names[tag_id]  # NOTE: will fail when we pass ~2.5M recommended objects
 
 
-class Recommendation(Base, CoreMixin):
+class Recommendation(ModelInterface):
     """Recommendation table."""
 
-    __tablename__ = 'recommendation'
-    __table_args__ = {'schema': schema}
-
     id = Column('id', BigInteger().with_variant(Integer(), 'sqlite'), primary_key=True, nullable=False)
     group_id = Column('group_id', Integer(), ForeignKey(RecommendationGroup.id), nullable=False)
     tag_id = Column('tag_id', Integer(), ForeignKey(RecommendationTag.id), nullable=False)
@@ -1269,12 +1186,9 @@ recommendation_group_user_index = Index('recommendation_group_user_index',
                                         Recommendation.group_id, Recommendation.user_id)
 
 
-class ModelType(Base, CoreMixin):
+class ModelType(ModelInterface):
     """Model type table."""
 
-    __tablename__ = 'model_type'
-    __table_args__ = {'schema': schema}
-
     id = Column('id', Integer(), primary_key=True, nullable=False)
     name = Column('name', Text(), unique=True, nullable=False)
     format = Column('format', Text(), nullable=False)
@@ -1300,12 +1214,9 @@ class ModelType(Base, CoreMixin):
             raise ModelType.NotFound(f'No model_type with name={name}') from error
 
 
-class Model(Base, CoreMixin):
+class Model(ModelInterface):
     """Model table."""
 
-    __tablename__ = 'model'
-    __table_args__ = {'schema': schema}
-
     id = Column('id', Integer().with_variant(BigInteger(), 'postgresql'), primary_key=True, nullable=False)
     type_id = Column('type_id', Integer(), ForeignKey(ModelType.id), nullable=False)
     name = Column('name', Text(), unique=True, nullable=False)
@@ -1340,8 +1251,186 @@ class Model(Base, CoreMixin):
             raise Model.NotFound(f'No model with name={name}') from error
 
 
+# ----------------------------------------------------------------------------------------------
+# Re-implementation of StreamKit models
+# We conform to the database schema but re-define under a common ModelInterface
+
+
+class Level(ModelInterface):
+    """A level relates a name and its identifier."""
+
+    id = Column('id', Integer, primary_key=True)
+    name = Column('name', String, unique=True, nullable=False)
+
+    columns = {
+        'id': int,
+        'name': str
+    }
+
+    class NotFound(NotFound):
+        """NotFound exception specific to Level."""
+
+    @classmethod
+    def from_name(cls, name: str, session: _Session = None) -> Level:
+        """Query by unique level `name`."""
+        try:
+            session = session or _Session()
+            return session.query(cls).filter(cls.name == name).one()
+        except NoResultFound as error:
+            raise Level.NotFound(f'No level with name={name}') from error
+
+
+class Topic(ModelInterface):
+    """A topic relates a name and its identifier."""
+
+    id = Column('id', Integer, primary_key=True)
+    name = Column('name', String, unique=True, nullable=False)
+
+    columns = {
+        'id': int,
+        'name': str
+    }
+
+    class NotFound(NotFound):
+        """NotFound exception specific to Topic."""
+
+    @classmethod
+    def from_name(cls, name: str, session: _Session = None) -> Topic:
+        """Query by unique topic `name`."""
+        try:
+            session = session or _Session()
+            return session.query(cls).filter(cls.name == name).one()
+        except NoResultFound as error:
+            raise Topic.NotFound(f'No topic with name={name}') from error
+
+
+class Host(ModelInterface):
+    """A host relates a name and its identifier."""
+
+    id = Column('id', Integer, primary_key=True)
+    name = Column('name', String, unique=True, nullable=False)
+
+    columns = {
+        'id': int,
+        'name': str
+    }
+
+    class NotFound(NotFound):
+        """NotFound exception specific to Host."""
+
+    @classmethod
+    def from_name(cls, name: str, session: _Session = None) -> Host:
+        """Query by unique host `name`."""
+        try:
+            session = session or _Session()
+            return session.query(cls).filter(cls.name == name).one()
+        except NoResultFound as error:
+            raise Host.NotFound(f'No host with name={name}') from error
+
+
+class Message(ModelInterface):
+    """A message joins topic, level, and host, with timestamp and an identifier for the message."""
+
+    id = Column('id', BigInteger().with_variant(Integer(), 'sqlite'), primary_key=True)
+    time = Column('time', DateTime(timezone=True), nullable=False)
+    topic_id = Column('topic_id', Integer(), ForeignKey(Topic.id), nullable=False)
+    level_id = Column('level_id', Integer(), ForeignKey(Level.id), nullable=False)
+    host_id = Column('host_id', Integer(), ForeignKey(Host.id), nullable=False)
+    text = Column('text', String(), nullable=False)
+
+    # Note: conditionally redefine for time-based partitioning
+    if config.provider in ('timescale', ):
+        # The primary key is (`time`, `topic_id`) NOT `id`.
+        # This is weird but important for automatic hyper-table partitioning
+        # on the `time` values for TimeScaleDB (PostgreSQL).
+        id = Column('id', BigInteger(),
+                    Sequence('message_id_seq', start=1, increment=1, schema=schema),
+                    CheckConstraint('id > 0', name='message_id_check'), nullable=False)
+        time = Column('time', DateTime(timezone=True), nullable=False, primary_key=True)
+        topic_id = Column('topic_id', Integer(), nullable=False, primary_key=True)
+
+    topic = relationship('Topic', backref='message')
+    level = relationship('Level', backref='message')
+    host  = relationship('Host', backref='message')
+
+    relationships = {'topic': Topic, 'level': Level, 'host': Host}
+    columns = {
+        'id': int,
+        'time': datetime,
+        'topic_id': int,
+        'level_id': int,
+        'host_id': int,
+        'text': str
+    }
+
+    class NotFound(NotFound):
+        """NotFound exception specific to Message."""
+
+
+if config.provider in ('timescale', ):
+    # NOTE: we use time-topic PK and need to index ID
+    message_id_index = Index('message_id_index', Message.id)
+    message_time_topic_index = None
+else:
+    message_id_index = None
+    message_time_topic_index = Index('message_time_topic_index', Message.time, Message.topic_id)
+
+
+# efficient filtering on host or level
+message_level_index = Index('message_level_index', Message.level_id)
+message_host_index = Index('message_host_index', Message.host_id)
+
+
+class Subscriber(ModelInterface):
+    """A subscriber relates a name and its identifier."""
+
+    id = Column('id', Integer, primary_key=True)
+    name = Column('name', String, unique=True, nullable=False)
+
+    columns = {
+        'id': int,
+        'name': str
+    }
+
+    class NotFound(NotFound):
+        """NotFound exception specific to Subscriber."""
+
+    @classmethod
+    def from_name(cls, name: str, session: _Session = None) -> Subscriber:
+        """Query by unique subscriber `name`."""
+        try:
+            session = session or _Session()
+            return session.query(cls).filter(cls.name == name).one()
+        except NoResultFound as error:
+            raise Subscriber.NotFound(f'No subscriber with name={name}') from error
+
+
+class Access(ModelInterface):
+    """Access tracks the last message received on a given topic for a given subscriber."""
+
+    subscriber_id = Column('subscriber_id', Integer, ForeignKey(Subscriber.id), nullable=False, primary_key=True)
+    topic_id = Column('topic_id', Integer, ForeignKey(Topic.id), nullable=False, primary_key=True)
+    time = Column('time', DateTime(timezone=True), nullable=False)
+
+    subscriber = relationship('Subscriber', backref='access')
+    topic = relationship('Topic', backref='access')
+
+    relationships = {'subscriber': Subscriber, 'topic': Topic}
+    columns = {
+        'subscriber_id': int,
+        'topic_id': int,
+        'time': datetime
+    }
+
+    class NotFound(NotFound):
+        """NotFound exception specific to Access."""
+
+
+# ----------------------------------------------------------------------------------------------
+
+
 # global registry of tables
-tables: Dict[str, Base] = {
+tables: Dict[str, ModelInterface] = {
     'facility': Facility,
     'user': User,
     'facility_map': FacilityMap,
@@ -1380,4 +1469,12 @@ indices: Dict[str, Index] = {
     'observation_object_index': observation_object_index,
     'observation_recorded_index': observation_recorded_index,
     'observation_source_object_index': observation_source_object_index,
+    'message_level_index': message_level_index,
+    'message_host_index': message_host_index,
 }
+
+# optionally defined depending on provider
+if config.provider in ('timescale', ):
+    indices['message_id_index'] = message_id_index
+else:
+    indices['message_time_topic_index'] = message_time_topic_index
